'use strict';

var fetchAssets = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(url) {
    var name, spinner, response, json;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            name = parseUrl(url).pathname.split('/').pop();
            spinner = ora('fetching ' + name).start();
            _context.next = 4;
            return fetch(url);

          case 4:
            response = _context.sent;
            _context.next = 7;
            return response.json();

          case 7:
            json = _context.sent;

            spinner.succeed('Fetched  ' + name);
            return _context.abrupt('return', json);

          case 10:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function fetchAssets(_x) {
    return _ref.apply(this, arguments);
  };
}();

var fetchSourceMapFile = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(url) {
    var name, spinner, response, text;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            name = parseUrl(url).pathname.split('/').pop();
            spinner = ora('Fetching ' + name).start();
            _context2.next = 4;
            return fetch(url);

          case 4:
            response = _context2.sent;
            _context2.next = 7;
            return response.text();

          case 7:
            text = _context2.sent;

            spinner.succeed('Fetched  ' + name);
            return _context2.abrupt('return', text);

          case 10:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function fetchSourceMapFile(_x2) {
    return _ref2.apply(this, arguments);
  };
}();

var collectSourceMaps = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(argv, url) {
    var _this = this;

    var _parseUrl, resource, protocol, port, base, assetsUrl, assets, sourceMapFileNames;

    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            if (!argv.mapFiles) {
              _context4.next = 4;
              break;
            }

            return _context4.abrupt('return', argv.mapFiles.map(function (mapFile) {
              var content = loadFile(mapFile);
              var name = path.basename(mapFile).replace('.map', '');

              var _consumeSourceMap = consumeSourceMap(content),
                  mapConsumer = _consumeSourceMap.mapConsumer;

              return {
                name: name,
                mapping: mapConsumer
              };
            }));

          case 4:
            if (!url) {
              _context4.next = 13;
              break;
            }

            _parseUrl = parseUrl(url), resource = _parseUrl.resource, protocol = _parseUrl.protocol, port = _parseUrl.port;
            base = resource + (port ? ':' + port : '');
            assetsUrl = protocol + '://' + base + '/assets/assets.json';
            _context4.next = 10;
            return fetchAssets(assetsUrl);

          case 10:
            assets = _context4.sent;
            sourceMapFileNames = getSourceMapFileNames(assets);
            return _context4.abrupt('return', Promise.all(sourceMapFileNames.map(function () {
              var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(fileName) {
                var urlToSourceMap, content, _consumeSourceMap2, mapConsumer, name;

                return regeneratorRuntime.wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        urlToSourceMap = protocol + '://' + base + '/assets/' + fileName;
                        _context3.next = 3;
                        return fetchSourceMapFile(urlToSourceMap);

                      case 3:
                        content = _context3.sent;
                        _consumeSourceMap2 = consumeSourceMap(content), mapConsumer = _consumeSourceMap2.mapConsumer;
                        name = path.basename(fileName).replace('.map', '');
                        return _context3.abrupt('return', {
                          name: name,
                          mapping: mapConsumer
                        });

                      case 7:
                      case 'end':
                        return _context3.stop();
                    }
                  }
                }, _callee3, _this);
              }));

              return function (_x5) {
                return _ref4.apply(this, arguments);
              };
            }())));

          case 13:
            throw new Error('No  sourcemaps to collect');

          case 14:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function collectSourceMaps(_x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}();

var runSourceMapResolver = function () {
  var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(argv) {
    var errorEvent, stackInfo, stack, url, sourceMaps;
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            errorEvent = JSON.parse(loadFile(argv.errorEventFile));
            stackInfo = errorEvent.stackInfo;
            stack = stackInfo.stack;
            url = stack[0].url;
            _context5.next = 6;
            return collectSourceMaps(argv, url);

          case 6:
            sourceMaps = _context5.sent;

            process.stdout.write(chalk.bold.red('\n' + stackInfo.name + ': ' + stackInfo.message + ' \n'));
            stack.forEach(function (frame) {
              return printOriginalPosition(sourceMaps, frame, argv.printSourceLine);
            });

          case 9:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));

  return function runSourceMapResolver(_x6) {
    return _ref5.apply(this, arguments);
  };
}();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/**
 * Copyright (c) 2016-present, NDLA.
 *
 * This source code is licensed under the GPLv3 license found in the
 * LICENSE file in the root directory of this source tree.
 *
 */

var fs = require('fs');
var path = require('path');
var util = require('util');
var SourceMap = require('source-map');
var chalk = require('chalk');
var parseUrl = require('parse-url');
var fetch = require('node-fetch');
var ora = require('ora');

function loadFile(fileName) {
  try {
    var filePath = path.resolve(fileName);
    var fileData = fs.readFileSync(filePath).toString();
    process.stdout.write(chalk.green('Loaded ' + fileName + ' \n'));
    return fileData;
  } catch (e) {
    process.stdout.write(chalk.bold.red('Error loading ' + fileName + '\n'));
    process.stdout.write(chalk.bold.red(util.format(e)));
    process.on('exit', function () {
      return process.exit(1);
    });
  }
  return undefined;
}

function consumeSourceMap(sourceMapData) {
  var mapConsumer = new SourceMap.SourceMapConsumer(sourceMapData);
  return {
    mapConsumer: mapConsumer
  };
}

function printSourceLine(mapConsumer, orgPos) {
  var src = mapConsumer.sourceContentFor(orgPos.source);
  var lines = src.split('\n');
  var line = lines[orgPos.line - 1];

  process.stdout.write(chalk.bold.green(line + ' \n'));
}

function printOriginalPosition(sourceMaps, frame, printSourceLineFlag) {
  var sourceMap = sourceMaps.find(function (map) {
    return frame.url.indexOf(map.name) !== -1;
  });

  if (sourceMap) {
    var orgPos = sourceMap.mapping.originalPositionFor({
      line: frame.line,
      column: frame.column
    });
    process.stdout.write(chalk.bold.red('  at ' + orgPos.name + ' ') + chalk.cyan('(' + orgPos.source + ':' + orgPos.line + ':' + orgPos.column + ') \n'));

    if (printSourceLineFlag) {
      printSourceLine(sourceMap.mapping, orgPos);
    }
  } else {
    process.stdout.write(chalk.bold.red('  at ' + frame.func + ' ') + chalk.cyan('(' + frame.url + ') \n'));
  }
}

function getSourceMapFileNames(assets) {
  return Object.keys(assets).map(function (key) {
    return assets[key];
  }).filter(function (name) {
    return name.indexOf('.js.map') > -1;
  });
}

module.exports = runSourceMapResolver;